/*****************************************************************************
 *									     *
 *     Copyright (c) 1986                  The Santa Cruz Operation, Inc.    *
 *									     *
 *     All rights reserved. No part of this program or publication may be    *
 *     reproduced, transmitted, transcribed, stored in a retrieval system,   *
 *     or translated into any language or computer language, in any form or  *
 *     by any means, electronic, mechanical, magnetic, optical, chemical,    *
 *     biological, or otherwise, without the prior written permission of:    *
 *									     *
 *	      The Santa Cruz Operation, Inc      (408) 425-7222	             *
 *	      500 Chestnut St, Santa Cruz, California 95060 USA              *
 *									     *
 *****************************************************************************/

#include <stdio.h>
#include <math.h>
#include "metalib.h"
#include "hpcpro.h"


/*
 * main -- need to do some device cleanup so we must include our own
 * 	main.
 */
main()
{
	setup_dev();

	/*
	 * this is the routine that will parse the metafile and call
	 * functions in this driver.
	 */
	meta_parse();

	/*
	 * clean up 
	 */
	close_dev();
}


/*
 * close_dev -  pick pen up and replace it
 */
close_dev()
{
	PEN_UP();
	REPLACE_PEN();
}

/*
 * setup_dev -- device initialization
 */
setup_dev()
{
	/*
	 * We need to use some sort of hand-shaking method, and
	 * since we cannot count on there being hardware hand-shaking,
	 * we must assume that XON-XOFF has been enabled.
	 */
	USE_XON_XOFF();
	INIT_PLOT();

	/*
	 * set up scaling so that VIRTCOORDs need not be scaled
	 */
	SET_HARD_CLIP();
	SET_SCALE_RANGE();
}



/*
 * polyline -- this routine is required if line and bar charts
 *	are to be drawn.  It calls read_point to get the points
 *	necessary to draw the polyline and calls line for each
 *	point.
 */
polyline(numpoints)
int numpoints;
{
	int ii;
	POINT thispoint;

	/* get the correct pen */
	SETCOLOR(line_attr.l_color);

	/* read the first point */
	read_point(&thispoint);

	/* first point is starting point */
	MOVE_TO(thispoint);

	/* draw those lines */
	for(ii=1; ii < numpoints; ii++)
	{
		read_point(&thispoint);
		LINE(thispoint);
	}
	PEN_UP();
}


/*
 * polygon -- polygon is called to draw the bars in a bar chart.
 *	It is much like polyline, but requires that the first point
 *	and the last point be connected with a line.  If this driver
 *	did some sort of a fill, polygons would be filled according
 *	to poly_attr.p_color.
 */
polygon(numpoints)
int	numpoints;
{
	int ii;
	POINT firstpoint;
	POINT thispoint;

	/* get the correct pen */
	SETCOLOR(poly_attr.p_color);

	/* first point is starting point */
	read_point(&firstpoint);
	MOVE_TO(firstpoint);

	/* draw those lines */
	for(ii=1; ii < numpoints; ii++)
	{
		read_point(&thispoint);
		LINE(thispoint);
	}

	/* connect up the first point */
	LINE(firstpoint);
	PEN_UP();
}


/*
 * graf_text -- this routine is called whenever text needs to be drawn.
 *	We must do all text alignment, and discover the various attributes
 *	of the requested text.
 */
graf_text(refpoint, finflag, len)
POINT *refpoint;
int finflag;
int len;
{
	char	buffer[MAXSTR];

	if (len > MAXSTR)
	{
		fprintf(stderr, "Overflowed string storage\n");
		exit(1);
	}

	/* routine provided by the metalib */
	read_string(buffer, len);

	/* set the text size */
	settsize();

	/* get the right pen */
	SETCOLOR(text_attr.t_color);

	/* move to the reference point */
	MOVE_TO(*refpoint);

	/* adjust the text based on the justification values */
	adjustext(buffer);

	/* draw that string */
	LABEL(buffer);
	PEN_UP();
}


/*
 * adjustext -- adjust the current pen position in preparation to put out
 *	text with the desired justification.
 */
adjustext(str)
char	*str;
{
	double	theta;

	/*
	 * Since PRO at this point only puts out vertical and horizontal
	 * text, it is not strictly necessary to compute angle using
	 * ArcTangent, but is included to demonstrate use of base vector.
	 */
	if (PX(text_attr.t_base_orient) == 0)
	{
		if (PY(text_attr.t_base_orient) < 0)
			theta = -M_PI/2;
		else
			theta = M_PI/2;
	}
	else
	{
		theta = atan((double)PY(text_attr.t_base_orient)/(double)PX(text_attr.t_base_orient));
		if (PX(text_attr.t_base_orient) < 0)
			theta += M_PI;
	}

	/*
	 * now that we have the angle of rotation...
	 */
	ROTATE_TEXT(theta);

	switch (text_attr.t_horiz_al)
	{
		case TXT_NORMAL:
		case TXT_LEFT:
		default:
			break;
		case TXT_RIGHT:
			charpos((double)-strlen(str), (double)0);
			break;
		case TXT_CENTER:
			charpos((double)-strlen(str)/2.0, (double)0);
			break;
	}

	switch (text_attr.t_vert_al)
	{
		case TXT_HALF:
			charpos((double)0, -MAG_HALF);
			break;
		default:
		case TXT_BASE:
			charpos((double)0, -MAG_BASE);
			break;
	}

}


/*
 * charpos -- position the pen to account for the passed values which
 *	correspond to the number of character positions to move.  The
 *	HPColorPro will move along the angle currently specified.
 */
charpos(x, y)
double	x, y;
{
	/* since lines are two characters high */
	y /= 2.0;

	CHAR_POSITION(x,y);
}
	

/*
 * settsize -- set the text size to the value set in text_attr.t_height
 */
settsize()
{
	SET_TEXT_SIZE(50.0*(float)text_attr.t_height/(float)VIRTMAX,100.0*(float)text_attr.t_height/(float)VIRTMAX);
}


/*
 * clos_arc -- used in drawing pie slices.
 */
clos_arc(cent, start, end, pcflag, virrad, startvang, endvang)
POINT *cent, *start, *end;
VIRTANG	startvang, endvang;
VIRTCOORD virrad;
int pcflag;
{
	POINT	tempoint;

	/* get the right pen */
	SETCOLOR(poly_attr.p_color);

	/* first draw the lines from the center */
	MOVE_TO(*cent);
	PX(tempoint) = PX(*end) + PX(*cent);
	PY(tempoint) = PY(*end) + PY(*cent);
	LINE(tempoint);

	MOVE_TO(*cent);
	PX(tempoint) = PX(*start) + PX(*cent);
	PY(tempoint) = PY(*start) + PY(*cent);
	LINE(tempoint);

	/*
	 * since we don't have, or do not know that we have
	 * a cartridge, go ahead and simulate the arc.
	 */
	simarc(cent, start, end, virrad, startvang, endvang);
	PEN_UP();
}



/* 
 * simarc-  this routine draws an arc without using the plotter
 *	commands for that purpose since we do not know if we have
 *	the cartridge.
 */
simarc(cent, start, end, radius, startvang, endvang)
POINT	*cent;
POINT	*start;
POINT	*end;
VIRTCOORD	radius;
VIRTANG		startvang, endvang;
{
	double	rstart, rend, ff;
	POINT	tempoint;

	/* move to start point */
	PX(tempoint) = PX(*start) + PX(*cent);
	PY(tempoint) = PY(*start) + PY(*cent);
	MOVE_TO(tempoint);

	/* determine angles in radians */
	rstart = (startvang * M_PI) / (MAXANGLE>>1);
	rend = (endvang * M_PI) / (MAXANGLE>>1);

	/* incrementally draw that circle */
	for(ff=rstart; ff < rend; ff += CIRCINC)
	{
		PX(tempoint) = (double)radius * cos(ff) + PX(*cent);
		PY(tempoint) = (double)radius * sin(ff) + PX(*cent);
		LINE(tempoint);
	}

	/* make sure that there is no gap in the arc, draw to end point */
	PX(tempoint) = PX(*end) + PX(*cent);
	PY(tempoint) = PY(*end) + PY(*cent);
	LINE(tempoint);
}


/*
 * polymark -- this function is used to place markers on the graph in
 *	line graphs.  The HPColorPro has no built in markers so we go
 *	ahead and use 1-6
 */
polymark(numpoints)
int numpoints;
{
	int ii;
	POINT tempoint;
	int	sym;

	/* get the right pen */
	SETCOLOR(mark_attr.m_color);

	/* account for somewhat strange marker range */
	sym = mark_attr.m_style;
	if (sym == -1)
		sym = 6;
	sym += '0';

	SET_SYMBOL_CHAR( sym);

	/* draw those markers */
	for(ii=0; ii < numpoints; ii++)
	{
		read_point(&tempoint);
		/* when symbol marking is on then MOVE_TO will set marker */
		MOVE_TO(tempoint);
	}

	STOP_SYMBOLS();
	PEN_UP();
}


/*
 * setmsize -- set the marker size using the current marker size in
 *	mark_attr.m_size.  Since markers are just text we set text
 *	size
 */
setmsize()
{
	SET_TEXT_SIZE(50.0*(float)mark_attr.m_size/(float)VIRTMAX,100.0*(float)mark_attr.m_size/(float)VIRTMAX);
}
